/*
 * ******************************************************************************************
 *
 * File		: apb_timer_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB Timer
 *
 * ******************************************************************************************
 */

#ifndef __APB_TIMER_DRIVER_H__
#define __APB_TIMER_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_timer.h"


// Definitions ------------------------------------------------------------------------------

/* Bit definitions for CTRL register */
#define TIMER_CTRL_EN          ((unsigned int) 0x00000001) /* TIMER CTRL: Enable                          */
#define TIMER_CTRL_SELEXTEN    ((unsigned int) 0x00000002) /* TIMER CTRL: Select external input as enable */
#define TIMER_CTRL_SELEXTCLK   ((unsigned int) 0x00000004) /* TIMER CTRL: Select external input as clock  */
#define TIMER_CTRL_IRQEN       ((unsigned int) 0x00000008) /* TIMER CTRL: Timer interrupt enable          */

/* Bit definitions for VALUE register */
#define TIMER_VALUE            ((unsigned int) 0xFFFFFFFF) /* TIMER VALUE: Current value                  */

/* Bit definitions for RELOAD register */
#define TIMER_RELOAD           ((unsigned int) 0xFFFFFFFF) /* TIMER RELOAD: Reload value                  */

/* Bit definitions for INTSTATUS register */
#define TIMER_INTSTATUS        ((unsigned int) 0x00000001) /* TIMER INTSTATUS: Get Timer interrupt status */

/* Bit definitions for INTCLEAR register */
#define TIMER_INTCLEAR         ((unsigned int) 0x00000001) /* TIMER INTCLEAR: Clear Timer interrupt       */


/* FlagStatus, ITStatus */
typedef enum
{
	RESET = 0,
	SET = !RESET
} FlagStatus, ITStatus;

/* FunctionalState */
typedef enum
{
	DISABLE = 0,
	ENABLE = !DISABLE
} FunctionalState;

/* ErrorStatus */
typedef enum
{
	ERROR = 0,
	SUCCESS = !ERROR
} ErrorStatus;


/* APB Timer interrupt typedef */
typedef FunctionalState TIMERInt_TypeDef;  /* ENABLE/DISABLE */

/* APB Timer external input typedef */
typedef enum
{
	TIMER_DISABLE = 0,  /* Disable */
	TIMER_EXTI_EN,      /* Enable to select input */
	TIMER_EXTI_CLK      /* Select external input as clock */
} TIMERExti_TypeDef;

/* APB Timer initialized structure definition */
typedef struct
{
	unsigned int Reload;             /* Reload value to set the current value */

	TIMERInt_TypeDef TIMER_Int;      /* Timer interrupt */

	TIMERExti_TypeDef TIMER_Exti;    /* Select timer external input */

} APB_TIMER_InitTypeDef;


//**********************************************
// Functions
//**********************************************

/**
  * @param	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	TIMER_InitStruct: point to APB_TIMER_InitTypeDef pointer
  * @return	None
  * @brief 	Initializes APB Timer device.
  */
void apb_timer_initialize(APB_TIMER_RegDef* TIMERx, APB_TIMER_InitTypeDef* TIMER_InitStruct);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Start APB Timer.
  */
void apb_timer_start(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Stop APB Timer.
  */
void apb_timer_stop(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Interrupt status
  *         @arg SET
  *         @arg RESET
  * @brief 	Get APB Timer interrupt status.
  */
ITStatus apb_timer_get_irq_status(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Clear APB Timer interrupt.
  */
void apb_timer_clear_irq(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Reload value
  * @brief 	Get APB Timer reload value.
  */
unsigned int apb_timer_get_reload(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	value: set the reload value
  * @return None
  * @brief 	Set APB Timer Reload value.
  */
void apb_timer_set_reload(APB_TIMER_RegDef* TIMERx, unsigned int value);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Current value
  * @brief 	Get APB Timer current value.
  */
unsigned int apb_timer_get_value(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	value: current value to be set
  * @return None
  * @brief 	Set APB Timer current value.
  */
void apb_timer_set_value(APB_TIMER_RegDef* TIMERx, unsigned int value);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Enable APB Timer interrupt.
  */
void apb_timer_enable_irq(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Disable APB Timer interrupt.
  */
void apb_timer_disable_irq(APB_TIMER_RegDef* TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief  Enable to select external input.
  */
void apb_timer_enable_select_ext(APB_TIMER_RegDef *TIMERx);

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief  Select external input clock.
  */
void apb_timer_select_ext_clock(APB_TIMER_RegDef *TIMERx);


#ifdef __cplusplus
}
#endif


#endif	/* __APB_TIMER_DRIVER_H__ */
