/*
 * ******************************************************************************************
 *
 * File		: apb_timer_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB Timer
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_timer_driver.h"


// Definitions ------------------------------------------------------------------------------

/**
  * @param	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	TIMER_InitStruct: point to APB_TIMER_InitTypeDef pointer
  * @return	None
  * @brief 	Initializes APB Timer device.
  */
void apb_timer_initialize(APB_TIMER_RegDef* TIMERx, APB_TIMER_InitTypeDef* TIMER_InitStruct)
{
	/* Initialize all registers to zero */
	TIMERx->CTRL = 0;
	TIMERx->VALUE = 0;
	TIMERx->RELOAD = 0;
	TIMERx->INTCLEAR = 0;

	/* VALUE register */
	TIMERx->VALUE = TIMER_InitStruct->Reload;

	/* RELOAD register */
	TIMERx->RELOAD = TIMER_InitStruct->Reload;

	/* Enable CTRL register timer */
	TIMERx->CTRL |= TIMER_CTRL_EN;

	/* If TIMER_Int is enabled */
	if(TIMER_InitStruct->TIMER_Int == ENABLE)
	{
		TIMERx->CTRL |= TIMER_CTRL_IRQEN;
	}

	/* Disable external input */
	if(TIMER_InitStruct->TIMER_Exti == TIMER_DISABLE)
	{
		/* Disable external input */
		TIMERx->CTRL &= ~(TIMER_CTRL_SELEXTEN |TIMER_CTRL_SELEXTEN );
	}

	/* Select external input */
	if(TIMER_InitStruct->TIMER_Exti == TIMER_EXTI_EN)
	{
		/* Enable to select external input */
		TIMERx->CTRL |= TIMER_CTRL_SELEXTEN;
	}
	else if(TIMER_InitStruct->TIMER_Exti == TIMER_EXTI_CLK)
	{
		/* Select external input as clock */
		TIMERx->CTRL |= TIMER_CTRL_SELEXTCLK;
	}
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Start APB Timer.
  */
void apb_timer_start(APB_TIMER_RegDef* TIMERx)
{
	TIMERx->CTRL |= TIMER_CTRL_EN;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Stop APB Timer.
  */
void apb_timer_stop(APB_TIMER_RegDef* TIMERx)
{
	TIMERx->CTRL &= ~TIMER_CTRL_EN;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Interrupt status
  *         @arg SET
  *         @arg RESET
  * @brief 	Get APB Timer interrupt status.
  */
ITStatus apb_timer_get_irq_status(APB_TIMER_RegDef* TIMERx)
{
	return (ITStatus)(TIMERx->INTSTATUS);
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Clear APB Timer interrupt.
  */
void apb_timer_clear_irq(APB_TIMER_RegDef* TIMERx)
{
	TIMERx->INTCLEAR = TIMER_INTCLEAR;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Reload value
  * @brief 	Get APB Timer reload value.
  */
unsigned int apb_timer_get_reload(APB_TIMER_RegDef* TIMERx)
{
	return TIMERx->RELOAD;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	value: set the reload value
  * @return None
  * @brief 	Set APB Timer Reload value.
  */
void apb_timer_set_reload(APB_TIMER_RegDef* TIMERx, unsigned int value)
{
	TIMERx->RELOAD = value;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return Current value
  * @brief 	Get APB Timer current value.
  */
unsigned int apb_timer_get_value(APB_TIMER_RegDef* TIMERx)
{
	return TIMERx->VALUE;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @param 	value: current value to be set
  * @return None
  * @brief 	Set APB Timer current value.
  */
void apb_timer_set_value(APB_TIMER_RegDef* TIMERx, unsigned int value)
{
	TIMERx->VALUE = value;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Enable APB Timer interrupt.
  */
void apb_timer_enable_irq(APB_TIMER_RegDef* TIMERx)
{
	TIMERx->CTRL |= TIMER_CTRL_IRQEN;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief 	Disable APB Timer interrupt.
  */
void apb_timer_disable_irq(APB_TIMER_RegDef* TIMERx)
{
	TIMERx->CTRL &= ~TIMER_CTRL_IRQEN;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief  Enable to select external input.
  */
void apb_timer_enable_select_ext(APB_TIMER_RegDef *TIMERx)
{
	TIMERx->CTRL = ~(TIMER_CTRL_SELEXTEN |TIMER_CTRL_SELEXTCLK );
	TIMERx->CTRL |= TIMER_CTRL_SELEXTEN;
}

/**
  * @param 	TIMERx: point to APB_TIMER_RegDef pointer
  * @return None
  * @brief  Select external input clock.
  */
void apb_timer_select_ext_clock(APB_TIMER_RegDef *TIMERx)
{
	TIMERx->CTRL = ~(TIMER_CTRL_SELEXTEN |TIMER_CTRL_SELEXTCLK );
	TIMERx->CTRL |= TIMER_CTRL_SELEXTCLK;
}
